/*
 * Project name:
      DAC click (Driving external Digital-to-Analog converter)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Description:
     This is a sample program which demonstrates the use of the Microchip's
     MCP4921 12-bit D/A converter with FT900 MCUs. This device accepts digital
     input (number from 0..4095) and transforms it to the output voltage,
     ranging from 0..Vref. In this example the DAC communicates with MCU
     through the SPI communication.
     Buttons P61 and P2 are used to change value sent to DAC.
 * Test configuration:
     MCU:             FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       Clicker 2 for FT900
                       http://www.mikroe.com/ft90x/clicker-2/
     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. Modules:    DAC click Board - ac:DAC_click
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
      - Place DAC click board at the mikroBUS socket 1.
 */


// DAC click module connections
sfr sbit SoftSpi_SDI at GPIO_PIN30_bit;
sfr sbit SoftSpi_SDO at GPIO_PIN29_bit;
sfr sbit SoftSpi_CLK at GPIO_PIN27_bit;
sfr sbit Chip_Select at GPIO_PIN28_bit;
// End DAC click module connections

// Variable declaration
int value;

// DAC increments (0..4095) --> output voltage (0..Vref)
void DAC_Output(unsigned int valueDAC) {
  char temp;

  Chip_Select = 0;                   // Select DAC chip

  // Send High Byte
  temp = (valueDAC >> 8) & 0x0F;     // Store valueDAC[11..8] to temp[3..0]
  temp |= 0x30;                      // Define DAC setting, see MCP4921 datasheet
  Soft_SPI_Write(temp);              // Send high byte via SPI

  // Send Low Byte
  temp = valueDAC;                   // Store valueDAC[7..0] to temp[7..0]
  Soft_SPI_Write(temp);              // Send low byte via SPI

  Chip_Select = 1;                   // Deselect DAC chip
}

void main(){
  // Set Chip Select pin as Output
  GPIO_Pin_Digital_Output(28);
  // Set PD0 and PD1 as digital input
  GPIO_Pin_Digital_Input(2);
  GPIO_Pin_Digital_Input(35);
  // Soft_SPI init
  Soft_SPI_Init();
  // Set initial DAC output value to the mid range
  value = 2048;
  while(1) {                                       // Endless loop
    if ((!GPIO_PIN35_bit) && (value < 4095)) {     // If PD1 button is pressed
      value = value + 5;                           //   increment value
      if (value >= 4095)
        value = 0;
    }
    else {
      if ((!GPIO_PIN2_bit) && (value > 0)) {        // If PD0 button is pressed
        value = value - 5;                          //   decrement value
        if (value <= 0)
          value = 4095;
      }
    }
    DAC_Output(value);                              // Send value to DAC chip
    Delay_ms(10);
  }
}