/*
 * Project name:
     EXPAND_click (Demonstration of mikroBUS EXPAND click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Description:
     This is code writes data to expander's PortB and reads it from expander's PortA.
     Result is shown via UART terminal.
 * Test configuration:
     MCU:             FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       Clicker 2 for FT900
                      http://www.mikroe.com/ft90x/clicker-2/
     Ext. modules:    ac:EXPAND_click - Expand click
     Oscillator:      CPU at full system clock, 100.000MHz
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Place EXPAND click board into the mikroBUS socket 1.
     - Place USB UART click board into the mikroBUS socket 2.
     - Use buttons to increment, decrement value
 */

#include <built_in.h>

// Port Expander module connections
sbit SPExpanderRST           at GPIO_PIN47_bit;
sbit SPExpanderCS            at GPIO_PIN28_bit;
// End Port Expander module connections

unsigned char i = 0, old_res = 0, res;


/**************************************************************************************************
* Init function
**************************************************************************************************/
void InitMCU(){
  SPIM1_Init();
  UART1_Init(56000);
}

/**************************************************************************************************
* MAIN PROGRAM
**************************************************************************************************/
char txt[10] = "0x";
char write;
void main() {
  InitMCU();

  GPIO_Pin_Digital_Output(28);
  GPIO_Pin_Digital_Output(47);
  GPIO_Pin_Digital_Input(35);
  GPIO_Pin_Digital_Input(2);

  SPExpanderRST = 1;

  Expander_Init(0);                      // Initialize Port Expander

  Expander_Set_DirectionPortA(0, 0x00);  // Set Expander's PORTA to be output

  Expander_Set_DirectionPortB(0,0xFF);   // Set Expander's PORTB to be input
  Expander_Set_PullUpsPortB(0,0xFF);     // Set pull-ups to all of the Expander's PORTB pins


  UART1_Write_Text("Start");
  write = 0xaa;
  while(1) {                             // Endless loop
    Expander_Write_PortA(0, write);      // Write PORTD to expander's PORTA
    res = Expander_Read_PortB(0);        // Read expander's PORTB

    if(!GPIO_PIN35_bit)
      write++;

    if(!GPIO_PIN2_bit)
      write--;

    if (res != old_res){
      ByteToHex(res, &txt+2);
      UART_Write_Text(txt);
      UART_Write_Text("\r\n");

      old_res = res;
      Delay_ms(100);
    }
  }
}

/**************************************************************************************************
* End of File
**************************************************************************************************/